<?php
declare(strict_types=1);

namespace App\Service\Remote\Edge;

use App\Entity\DTO\QRCode\QRCode;
use App\Entity\DTO\QRCode\QRCodeMoney;
use App\Entity\DTO\QRCode\QRCodePaymentIdentification;
use App\Entity\DTO\QRCode\QRCodeQROptions;
use App\Entity\Remote\Usetreno\Edge\EdgeQRCode;
use App\Entity\Remote\Usetreno\Edge\EdgeQRCodeMoney;
use App\Entity\Remote\Usetreno\Edge\EdgeQRCodePaymentIdentification;
use App\Entity\Remote\Usetreno\Edge\EdgeQRCodeQROptions;
use App\Service\Remote\Edge\Exceptions\MissingParameterException;

class QRCodeEntityConverter
{
    public function __construct() { }

    public function convert(QRCode $code): EdgeQRCode {
        return new EdgeQRCode(
            $code->iban,
            $code->dueDate->format('Y-m-d'),
            $code->message,
            $this->convertMoney($code->money),
            $this->convertCodeOptions($code->qrOptions),
            $this->convertIdentification($code->paymentIdentification)
        );
    }

    protected function convertMoney(QRCodeMoney $money): EdgeQRCodeMoney
    {
        return new EdgeQRCodeMoney(
            $money->amount,
            $money->currency
        );
    }

    protected function convertCodeOptions(QRCodeQROptions $options): EdgeQRCodeQROptions
    {
        return new EdgeQRCodeQROptions(
            $options->scale,
            $options->margin
        );
    }

    protected function convertIdentification(?QRCodePaymentIdentification $ident): EdgeQRCodePaymentIdentification {
        // OMG: proc? symboly v ostatnich statech nejsou, prijde me lepsi posilat NULL, nebo empty string ;-)
        return match ($ident) {
            null => new EdgeQRCodePaymentIdentification(
                "0",
                "0",
                "0"
            ),
            default => new EdgeQRCodePaymentIdentification(
                $ident->variableSymbol ?? "0",
                $ident->specificSymbol ?? "0",
                $ident->constantSymbol ?? "0"
            )
        };
    }
}
